/*
 * Resolve ID's by using the gnome xml2 library
 */

#include <unistd.h>
#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>

#include <libxml/xmlversion.h>
#include <libxml/tree.h>
#ifdef LIBXML_CATALOG_ENABLED
#include <libxml/catalog.h>
#endif

#define TRACE if (verbose) printf

static void usage(const char *name)
{
  printf("Usage: %s [options] -- ID [ID ...]\n", name);
  printf("\tOptions:\n");
  printf("\t-v | --verbose: verbose mode\n");
  printf("\t-c | --catalog file: load the catalog file\n");
  printf("\t-n | --nosystem: the SGML_CATALOG_FILES catalogs are not loaded\n");
}
  

int
main(int argc, char **argv)
{
  int c;
  int verbose = 0;
  int use_system_catalog = 1;
  int failed = 0;
  const char* program = argv[0];
  static struct option long_options[] =
  {
    {"catalog", 1, 0, 'c'},
    {"verbose", 0, 0, 'v'},
    {"nosystem", 0, 0, 'n'},
    {0,0,0,0}
  };

  while (1)
  {
    int option_index;
    c = getopt_long (argc, argv, "nvc:", long_options, &option_index);

    if (c == -1) {
      break;
    }

    switch (c)
    {
      case 'c':
        TRACE("Loading catalog %s\n", optarg); 
        xmlLoadCatalog(optarg);
        break;
      case 'v':
        verbose++;
        break;
      case 'n':
        use_system_catalog = 0;
        break;
      default:
        usage(program);
        exit(1);
        break;
    }
  }

  /* At least one ID to resolve is needed */
  if (optind == argc) {
    usage(program);
    exit(1);
  }
  argc -= optind;
  argv += optind;

  /* System Catalog loading */
  if (use_system_catalog)
  {
    const char *catalogs = getenv("SGML_CATALOG_FILES");
    if (catalogs == NULL) {
      fprintf(stderr, "Variable $SGML_CATALOG_FILES not set\n");
    } else {
      TRACE("Loading system catalogs from %s\n", catalogs); 
      xmlLoadCatalogs(catalogs);
    }
  }

  while (argc--) {
    const xmlChar *ressource;
    const xmlChar *id = (const xmlChar *)(*argv++);

    /* Let's try both PUBLIC and SYSTEM */
    ressource = xmlCatalogGetPublic(id);
    if (ressource == NULL) {
      ressource = xmlCatalogGetSystem(id);
    }
    printf ("Resolved: %s\n", ressource);
    failed += (ressource == NULL);
  }
  return (failed) ? -1 : 0;
}

